<?php

namespace NodeExpireExtras;

interface IntervalEnum {
  const Undefined = 0;
  const Day = 1;
  const Week = 2;
  const Month = 3;
  const Year = 4;
}

interface UpdateExpireStartEnum {
  const ExpiryDate = 0;
  const CurrentDate = 1;
}

interface DateStrInterface {
  public function GetDateStr();
}

class DateStr implements DateStrInterface  {

  /**
   * Multiplier.
   *
   * @var int
   */
  protected $multiplier;

  /**
   * Interval.
   *
   * @var int
   */
  protected $interval;

  function __construct($multiplier, $interval) {
    $this->multiplier = $multiplier;
    $this->interval = $interval;
  }

  public function getDateStr() {

    // Sign
    $dateStr = "+ ";
    if ($this->interval < 0 ) {
      $dateStr="- ";
    }

    // Value
    $absMultiplier = abs($this->multiplier);
    $dateStr .= $absMultiplier . " ";

    // Interval
    switch ($this->interval) {
      case IntervalEnum::Day:
        $dateStr .= 'day';
        break;
      case IntervalEnum::Week:
        $dateStr .= 'week';
        break;
      case IntervalEnum::Month:
        $dateStr .= 'month';
        break;
      case IntervalEnum::Year:
        $dateStr .= 'year';
        break;
      default:
        // Year
        $dateStr .= "year";
        break;
    }

    if ($absMultiplier > 1) {
      $dateStr .= "s";
    }

    return $dateStr;

  }

}

class NodeExpireDateHelper {

  public static function isInteger($input){
    return(ctype_digit(strval($input)));
  }

  public static function extendExpireDate($nodeType, $expire) {

    if (empty($expire)) {
      return ''; // Special case, date is not set, keep it
    }

    $configHandler = new NodeTypeConfigHandler($nodeType);

    $start = REQUEST_TIME;

    // Period
    switch ($configHandler->getUpdateExpiryStart()) {
      case UpdateExpireStartEnum::ExpiryDate:
        $start = strtotime($expire);
        break;
      case UpdateExpireStartEnum::CurrentDate:
        // Do nothing
        break;
      default:
        // Current Date
        // Do nothing
        break;
    }

    $dateStr = new DateStr($configHandler->getUpdateExpiryMultiplier(),
      $configHandler->getUpdateExpiryInterval());
    $str = $dateStr->getDateStr();

    $expireDate = date(NODE_EXPIRE_FORMAT, strtotime($str, $start));

    return $expireDate;
  }
}

class NodeTypeConfigHandler  {

  /**
   * Node type.
   *
   * @var string
   */
  protected $nodeType;

  /**
   * Period.
   *
   * @var array
   */
  protected $nodeTypeConfigs;

  function __construct($nodeType) {
    $this->nodeType = $nodeType;
    $this->loadConfig();
  }

  protected function loadConfig() {
    $this->nodeTypeConfigs = variable_get('node_expire_ntypes', array());
  }

  protected function getNodeTypeConfigDefault() {
    $configDefault = array(
      'update_expiry_start'  => 1, // Current date
      'update_expiry_interval' => 4, // Year
      'update_expiry_multiplier' => 1,
    );

    return $configDefault;
  }

  // Single node type config
  public function getNodeTypeConfig() {
    if ((isset($this->nodeTypeConfigs[$this->nodeType]))
      and ($nTypeConfig = $this->nodeTypeConfigs[$this->nodeType])) {
      return $nTypeConfig;
    } else {
      return $this->getNodeTypeConfigDefault();
    }
  }

  public function getUpdateExpiryStart() {
    $config = $this->getNodeTypeConfig();
    if (isset($config['update_expiry_start'])) {
      return $config['update_expiry_start'];
    } else {
      return 1;
    }
  }

  public function getUpdateExpiryInterval() {
    $config = $this->getNodeTypeConfig();
    if (isset($config['update_expiry_interval'])) {
      return $config['update_expiry_interval'];
    } else {
      return 4;
    }
  }

  public function getUpdateExpiryMultiplier() {
    $config = $this->getNodeTypeConfig();
    if (isset($config['update_expiry_multiplier'])) {
      return $config['update_expiry_multiplier'];
    } else {
      return 1;
    }
  }

}
